﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.Net;
using System.IO;
using System.Xml;
using System.ServiceModel.Syndication;
using System.Windows.Media.Imaging;
using System.Threading;
using System.Diagnostics;
using System.Windows.Threading;

namespace Async.Workers
{
	class TheConfigureAwaitWay : IDoWork
	{
		private static readonly Uri FlickrRss = new Uri("http://api.flickr.com/services/feeds/photos_public.gne");

		private CancellationTokenSource _cancellationTokenSource;
		private Dispatcher _dispatcher;

		public async void DoWork(ObservableCollection<FlickrImage> images)
		{
			_cancellationTokenSource = new CancellationTokenSource();
			_dispatcher = Dispatcher.FromThread(Thread.CurrentThread);

			await Task.Run(async () =>
			{
				WebClient wc = new WebClient();

				while(!_cancellationTokenSource.IsCancellationRequested)
				{
					try
					{
						string str = await wc.DownloadStringTaskAsync(FlickrRss).ConfigureAwait(false);
						StringReader sr = new StringReader(str);

						XmlReader reader = XmlReader.Create(sr);
						SyndicationFeed feed = SyndicationFeed.Load(reader);

						Parallel.ForEach(feed.Items, async (item) =>
						{
							try
							{
                                FlickrImage fi = await DownloadItemAsync(item, _cancellationTokenSource.Token).ConfigureAwait(false);
								_dispatcher.Invoke(() =>
								{
									images.Add(fi);
								});
							}
							catch(WebException)
							{
								Debug.WriteLine("Web error/cancel");
							}
						});

						await Task.Delay(5000, _cancellationTokenSource.Token);
					}
					catch(WebException)
					{
						Debug.WriteLine("Cancelled");
					}
					catch(TaskCanceledException)
					{
						Debug.WriteLine("Task cancelled");
					}
				}

				Debug.WriteLine("Cancelled");
			});

		}

        private async Task<FlickrImage> DownloadItemAsync(SyndicationItem item, CancellationToken cancellationToken)
		{
            Debug.WriteLine("DownloadItemAsync: " + item.Title.Text);

			Uri uri = (from link in item.Links where link.RelationshipType == "enclosure" select link.Uri).FirstOrDefault();

			WebClient wc = new WebClient();
			cancellationToken.Register(() => wc.CancelAsync());
			byte[] data = await wc.DownloadDataTaskAsync(uri).ConfigureAwait(false);

			if(cancellationToken.IsCancellationRequested)
				return null;

			MemoryStream ms = new MemoryStream(data);
			BitmapImage bi = null;
			_dispatcher.Invoke(() =>
			{
				bi = new BitmapImage();
				bi.BeginInit();
				bi.StreamSource = ms;
				bi.EndInit();
			});

			FlickrImage fi = new FlickrImage();
			fi.Image = bi;
			fi.Title = item.Title.Text;

			return fi;
		}

		public void Cancel()
		{
			_cancellationTokenSource.Cancel();
		}
	}
}
